@@ -61,8 +61,10 @@ class tdcoa:
     systemspath = ''
     filesetpath = ''
     outputpath = ''
-    version = "0.3.9.6.9"
-    skip_dbs = False
+    version = "0.3.9.7.3"
+    skip_dbs = False    # skip ALL dbs connections / executions
+    manual_run = False  # skip dbs executions in execute_run() but not upload_to_transcend()
+                        # also skips /*{{save:}}*/ special command
 
     # dictionaries
     secrets = {}
@@ -177,7 +179,7 @@ class tdcoa:
 
         # ensure all required configuration files are present:
         self.utils.log('checking core config files')
-        startfiles = ['secrets.yaml','config.yaml','source_systems.yaml','run_gui.py','run_cmdline.py','run_cmdline']
+        startfiles = ['secrets.yaml','config.yaml','source_systems.yaml','run_gui.py','run_gui','run_cmdline.py','run_cmdline']
         startfilecontent = ''
         for startfile in startfiles:
             startfile_src = os.path.join(os.path.dirname(tdcsm.__file__), startfile)
@@ -262,12 +264,13 @@ class tdcoa:
         self.settings = configyaml['settings']
         self.utils.check_setting(self.settings,
                            required_item_list=['githost', 'gitfileset', 'gitmotd', 'localfilesets',
-                                               'run_non_fileset_folders'],
+                                               'run_non_fileset_folders', 'gui_show_dev_filesets'],
                            defaults=['https://raw.githubusercontent.com/tdcoa/sql/master/',
                                      'filesets.yaml',
                                      'motd.txt',
                                      '{download}/filesets.yaml',
-                                     'True'])
+                                     'True',
+                                     'False'])
 
         # add skip_dbs back in as silent (unlisted) option
         self.skip_dbs = False
@@ -343,9 +346,9 @@ class tdcoa:
                 # todo add default dbsversion and collection
                 self.utils.check_setting(self.systems[sysname],
                                    required_item_list=['active', 'siteid', 'use', 'host', 'username', 'password',
-                                                       'logmech', 'driver', 'encryption'],
+                                                       'logmech', 'driver', 'encryption','manual_run'],
                                    defaults=['True', 'siteid123', 'unknown', 'customer.host.missing.com',
-                                             'username_missing', 'password_missing', '', 'sqlalchemy', ''])
+                                             'username_missing', 'password_missing', '', 'sqlalchemy', '', 'False'])
 
                 if 'connectionstring' not in sysobject:
                     if sysobject['logmech'].strip() == '':
@@ -518,7 +521,7 @@ class tdcoa:
 
                             # match downloaded file to fileset object so that we can compare collection & dbsversion
                             # note: fileset may not always have dbsversion or collection listed. always copy if thats true
-                            for file_object, file_values in self.filesets['feature_usage']['files'].items():
+                            for file_object, file_values in self.filesets['customer_data_space']['files'].items():
                                 if downloaded_file == file_values['gitfile'].split('/')[-1]:
                                     if 'dbsversion' in file_values:
                                         if sysobject['dbsversion'] not in file_values['dbsversion']:
@@ -538,6 +541,7 @@ class tdcoa:
         self.utils.log('\ndone!')
         self.utils.log('time', str(dt.datetime.now()))
 
+
     def apply_override(self, override_folder='', target_folder=''):
         self.utils.log('applying file override')
 
@@ -565,6 +569,7 @@ class tdcoa:
                 self.utils.log('these files replace any matching filename, regardless of subfolder location')
                 for file in files:
                     if file == 'config.yaml' or file == 'secrets.yaml':
+                        # TODO: add  source_systems.yaml -- or, maybe can be removed?
                         copyops[os.path.join(self.approot, file)] = os.path.join(override_folder, file)
                         reloadconfig = True
                         self.utils.log('  config file found, reload imminent', file)
@@ -608,6 +613,7 @@ class tdcoa:
             self.reload_config()
         self.utils.log('\napply override complete!')
 
+
     def prepare_sql(self, sqlfolder='', override_folder=''):
         self.utils.log('prepare_sql started', header=True)
         self.utils.log('time', str(dt.datetime.now()))
@@ -624,7 +630,7 @@ class tdcoa:
         self.utils.log('empty run folder entirely')
         self.utils.recursively_delete_subfolders(os.path.join(self.approot, self.folders['run']))
 
-        # iterate all system level folders in "run" folder...
+        # iterate all system level folders in "sql" folder...
         for sysfolder in os.listdir(os.path.join(self.approot, self.folders['sql'])):
             if os.path.isdir(os.path.join(self.approot, self.folders['sql'])):
                 self.utils.log('\n' + '-' * self.utils.logspace)
@@ -673,7 +679,7 @@ class tdcoa:
                                 sqlpath = os.path.join(self.approot, self.folders['sql'], sysfolder, setfolder)
                                 runpath = os.path.join(self.approot, self.folders['run'], sysfolder)
 
-                                # todo combine into single makedirs statement instead of 2 mkdir
+                                # TODO: combine into single makedirs statement instead of 2 mkdir
                                 if not os.path.isdir(runpath):
                                     self.utils.log('  creating system folder', runpath)
                                     os.mkdir(runpath)
@@ -1125,6 +1131,175 @@ class tdcoa:
         shutil.move(runlogsrc, runlogdst)
 
 
+    def make_customer_files(self, name=''):
+        self.utils.log('make_customer_files started', header=True)
+        self.utils.log('time', str(dt.datetime.now()))
+
+        # at this point, we make the assumption that everything in the "run" directory is valid
+
+        # make output directory for execution output and other collateral
+        runpath = os.path.join(self.approot, self.folders['run'])
+        outputpath = self.make_output_folder(name)
+
+        # create hidden file containing last run's output -- to remain in the root folder
+        with open(os.path.join(self.approot, '.last_run_output_path.txt'), 'w') as lastoutput:
+
+            # convert to relative path in case of system change (i.e. switching laptops)
+            lastoutput.write(outputpath[outputpath.find(self.folders['output']):])
+
+        self.utils.log('save location of last-run output folder to hidden file')
+        self.utils.log('last-run output', outputpath)
+
+        self.utils.log('creating upload manifest file')
+        with open(os.path.join(outputpath, 'upload-manifest.json'), 'a') as manifest:
+            manifest.write('{"entries":[ ')
+
+        # loop through systems
+        for sysname in os.listdir(runpath):
+            sysfolder = os.path.join(runpath, sysname)
+            if os.path.isdir(sysfolder):
+
+                # iterate system folders  -- must exist in config.yaml!
+                if sysname not in self.systems:
+                    self.utils.log('SYSTEM NOT FOUND IN CONFIG.YAML', sysname, warning=True)
+
+                else:
+
+                    # iterate file set folders -- ok to NOT exist, depending on setting
+                    for setname in os.listdir(sysfolder):
+                        setfolder = os.path.join(sysfolder, setname)
+                        if os.path.isdir(setfolder):
+
+                            if setname not in self.systems[sysname]['filesets'] and str(
+                                    self.settings['run_non_fileset_folders']).strip().lower() != 'true':
+                                self.utils.log('-' * self.utils.logspace)
+                                self.utils.log('WARNING!!!\nfileset does not exist', setname)
+                                self.utils.log(' AND setting "run_non_fileset_folders" is not "True"')
+                                self.utils.log(' Skipping folder', setname)
+
+                            else:
+                                self.utils.log('SYSTEM:  %s   FILESET:  %s' % (sysname, setname), header=True)
+                                workpath = setfolder
+                                #outputfo = os.path.join(outputpath, sysname, setname)
+
+                                self.utils.log('work (sql) path', workpath)
+                                #self.utils.log('output path', outputfo)
+
+                                # collect all prepared sql files, place in alpha order
+                                coasqlfiles = []
+                                for coafile in os.listdir(workpath):
+                                    if coafile[:1] != '.' and coafile[-8:] == '.coa.sql':
+                                        self.utils.log('found prepared sql file', coafile)
+                                        coasqlfiles.append(coafile)
+
+                                coasqlfiles.sort()
+
+                                if len(coasqlfiles) == 0:
+                                    self.utils.log('no .coa.sql files found in\n  %s' % workpath, warning=True)
+
+                                else:
+                                    self.utils.log('all sql files alpha-sorted for execution consistency')
+                                    self.utils.log('sql files found', str(len(coasqlfiles)))
+
+
+
+                                    # create our upload-manifest, 1 manifest per fileset
+
+                                    manifestdelim = '\n '
+
+
+                                    # loop thru all sql files:
+                                    for coasqlfile in sorted(coasqlfiles):
+                                        self.utils.log('\nOPENING SQL FILE', coasqlfile)
+                                        with open(os.path.join(workpath, coasqlfile), 'r') as coasqlfilehdlr:
+                                            sqls = coasqlfilehdlr.read()  # all sql statements in a sql file
+
+                                        sqlcnt = 0
+                                        for sql in sqls.split(';'):  # loop thru the individual sql statements
+                                            sqlcnt += 1
+
+                                            if sql.strip() == '':
+                                                self.utils.log('null statement, skipping')
+
+                                            else:
+                                                self.utils.log('\n---- SQL #%i' % sqlcnt)
+
+                                                # pull out any embedded SQLcommands:
+                                                sqlcmd = self.utils.get_special_commands(sql)
+
+
+                                                if len(sqlcmd) == 0:
+                                                    self.utils.log('no special commands found')
+
+
+                                            file_ext = ".manual.coa.sql"
+                                            manual_sqlfile_name = sysname + "." + setname + file_ext
+                                            manual_sqlfile_path= os.path.join(outputpath,manual_sqlfile_name)
+                                            manual_save_file_name = ""
+                                            if 'save' in sqlcmd or 'load' in sqlcmd or 'call' in sqlcmd:
+                                                manual_save_file_name=sysname + "." + setname + "." + sqlcmd['save']
+
+                                                with open(manual_sqlfile_path, 'a') as manual_file:
+                                                    manual_file.write(
+                                                        "------------------------------------------------------------- \n")
+                                                    manual_file.write(
+                                                        "/* Save the result of the below sql as a csv file with name: " +
+                                                        manual_save_file_name + "*/ \n")
+                                                    manual_file.write(
+                                                        "------------------------------------------------------------- \n")
+                                                    manual_file.write(sql + ";")
+                                                    manual_file.write("\n\n")
+                                            else:
+                                                with open(manual_sqlfile_path, 'a') as manual_file:
+                                                    manual_file.write(sql + ";")
+                                                    manual_file.write("\n\n")
+
+                                            if 'load' in sqlcmd:  # add to manifest
+                                                self.utils.log(
+                                                            'file marked for loading to Transcend, adding to upload-manifest.json')
+                                                if 'call' not in sqlcmd:
+                                                    sqlcmd['call'] = ''
+                                                manifestdelim = '\n,'
+                                                manifest_entry = '{"file": "%s",  "table": "%s",  "call": "%s"}%s' % (
+                                                     manual_save_file_name, sqlcmd['load'],
+                                                    sqlcmd['call'],manifestdelim)
+
+
+                                                manifest_filepath=os.path.join(outputpath, 'upload-manifest.json')
+                                                manifest=open(manifest_filepath,'a')
+                                                manifest.write(manifest_entry)
+                                                self.utils.log('Manifest updated',str(manifest_entry).replace(',', ',\n'))
+                                                manifest.close()
+
+                                self.utils.recursive_delete(workpath)
+
+        manifest = open(os.path.join(outputpath, 'upload-manifest.json'), 'a')
+        manifest.write("\n  ]}")
+        self.utils.log('closing out upload-manifest.json')
+        manifest.close()
+        # Below logic is to remove the comma (,) at the end of the last element in the manifest json file.
+        with open(os.path.join(outputpath, 'upload-manifest.json'),'r') as manifest_file:
+            manifest_content=manifest_file.read()
+            manifest_content=manifest_content.replace(",\n  ]}","\n  ]}")
+        with open(os.path.join(outputpath, 'upload-manifest.json'), 'w') as manifest_file:
+            manifest_file.write(manifest_content)
+        # also COPY a few other operational files to output folder, for ease of use:
+        self.utils.log('-' * self.utils.logspace)
+        self.utils.log('post-processing')
+        for srcpath in [os.path.join(self.approot, '.last_run_output_path.txt'),
+                        self.configpath, self.filesetpath]:
+            self.utils.log('copy to output folder root, for ease of use: \n  %s' % srcpath)
+            dstpath = os.path.join(outputpath, os.path.basename(srcpath))
+            shutil.copyfile(srcpath, dstpath)
+
+        self.utils.log('\ndone!')
+        self.utils.log('time', str(dt.datetime.now()))
+
+        # after logging is done, move the log file too...
+        runlogsrc = os.path.join(self.approot, self.folders['run'], 'runlog.txt')
+        runlogdst = os.path.join(outputpath, 'runlog.txt')
+        shutil.move(runlogsrc, runlogdst)
+        self.utils.log('make_customer_files Completed', header=True)
 
     def upload_to_transcend(self, _outputpath=''):
         self.utils.bufferlogs = True
